@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2019, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2020, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -199,6 +199,11 @@ struct text_wrap : Config<bool> {
     static Value defaultValue() { return true; }
 };
 
+struct text_tab_width : Config<int> {
+    static QString name() { return "text_tab_width"; }
+    static Value defaultValue() { return 8; }
+};
+
 struct activate_closes : Config<bool> {
     static QString name() { return "activate_closes"; }
     static Value defaultValue() { return true; }
