@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2019, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2020, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -42,6 +42,7 @@
 #include "gui/pixelratio.h"
 #include "gui/screen.h"
 #include "gui/tabicons.h"
+#include "gui/traymenu.h"
 #include "gui/windowgeometryguard.h"
 #include "item/serialize.h"
 #include "platform/platformnativeinterface.h"
@@ -754,14 +755,26 @@ private:
 
         const auto className = widget->metaObject()->className();
 
-        const auto widgetName = QString("%1:%2")
+        auto widgetName = QString("%1:%2")
                 .arg(widget->objectName(), className);
 
         const auto window = widget->window();
         if (window && widget != window) {
-            return widgetName
-                + QString(" in %1:%2")
-                    .arg(window->objectName(), window->metaObject()->className());
+            widgetName.append(
+                QString(" in %1:%2")
+                    .arg(window->objectName(), window->metaObject()->className())
+            );
+        }
+
+        auto parent = widget->parentWidget();
+        while (parent) {
+            if ( parent != window && !parent->objectName().startsWith("qt_") ) {
+                widgetName.append(
+                    QString(" in %1:%2")
+                        .arg(parent->objectName(), parent->metaObject()->className())
+                );
+            }
+            parent = parent->parentWidget();
         }
 
         return widgetName;
@@ -1285,6 +1298,25 @@ QString ScriptableProxy::browserRemoveRows(const QString &tabName, QVector<int>
     return QString();
 }
 
+void ScriptableProxy::browserMoveSelected(int targetRow)
+{
+    INVOKE2(browserMoveSelected, (targetRow));
+
+    QVector<int> selectedRows;
+    const QList<QPersistentModelIndex> selected = selectedIndexes();
+    if ( selected.isEmpty() )
+        return;
+
+    ClipboardBrowser *c = m_wnd->browserForItem(selected.first());
+    if (c == nullptr)
+        return;
+
+    QModelIndexList indexes;
+    for (const auto index : selected)
+        indexes.append(index);
+    c->move(indexes, targetRow);
+}
+
 void ScriptableProxy::browserEditRow(const QString &tabName, int arg1)
 {
     INVOKE2(browserEditRow, (tabName, arg1));
@@ -1327,6 +1359,33 @@ int ScriptableProxy::findTabIndex(const QString &arg1)
     return m_wnd->findTabIndex(arg1);
 }
 
+int ScriptableProxy::menuItems(const QVector<QVariantMap> &items)
+{
+    INVOKE(menuItems, (items));
+
+    TrayMenu menu;
+    menu.setObjectName("CustomMenu");
+
+    const auto addMenuItems = [&](const QString &searchText) {
+        menu.clearClipboardItems();
+        for (const QVariantMap &data : items) {
+            const QString text = getTextData(data);
+            if ( text.contains(searchText, Qt::CaseInsensitive) )
+                menu.addClipboardItemAction(data, true);
+        }
+    };
+    addMenuItems(QString());
+
+    connect(&menu, &TrayMenu::searchRequest, addMenuItems);
+
+    const QPoint pos = QCursor::pos();
+    QAction *act = menu.exec(pos);
+    if (act == nullptr)
+        return -1;
+
+    return items.indexOf(act->data().toMap());
+}
+
 void ScriptableProxy::openActionDialog(const QVariantMap &arg1)
 {
     INVOKE2(openActionDialog, (arg1));
@@ -1974,21 +2033,12 @@ void ScriptableProxy::setTitle(const QString &title)
 {
     INVOKE2(setTitle, (title));
 
-    const QString sessionName = qApp->property("CopyQ_session_name").toString();
     if (title.isEmpty()) {
-        m_wnd->setWindowTitle("CopyQ");
-        m_wnd->setTrayTooltip("CopyQ");
+        m_wnd->setWindowTitle(QString());
+        m_wnd->setTrayTooltip(QGuiApplication::applicationDisplayName());
     } else {
-        const QString fullTitle = sessionName.isEmpty()
-                ? tr("%1 - CopyQ",
-                     "Main window title format (%1 is clipboard content label)")
-                  .arg(title)
-                : tr("%1 - %2 - CopyQ",
-                     "Main window title format (%1 is clipboard content label, %2 is session name)")
-                  .arg(title, sessionName);
-
-        m_wnd->setWindowTitle(fullTitle);
-        m_wnd->setTrayTooltip(fullTitle);
+        m_wnd->setWindowTitle(title);
+        m_wnd->setTrayTooltip(title);
     }
 }
 
